package Search::Glimpse::Server;

use base 'Exporter';

our $VERSION = '0.01';

!0;

=encoding UTF-8

=head1 NAME

Search::Glimpse::Server - Perl interface to start Glimpse server.

=head1 SYNOPSIS

  use Search::Glimpse::Server

=head1 DESCRIPTION




=head1 SEE ALSO

perl(1)

=head1 AUTHOR

Alberto Manuel Brandão Simões, E<lt>ambs@cpan.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2011 by Alberto Manuel Brandão Simões

=cut

